/*globals window, define*/
/*jslint nomen:true, unparam: true*/

/*
 * @copyright       © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/taxus/inc/taxus-tools
 *
 * @revision        $Revision: 1.3 $
 */

define("de_epages/taxus/inc/taxus-tools", [
    "jquery",
    "ep",

    "jquery/ui/autocomplete",
    "ep/validate",
    "ep/fn/busy"
], function ($, ep) {
    'use strict';

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Type}
     *                              {EmptyEntry}
     */

    var getAddressFromZip = function (options) {
        var $zip = options.zip,
            insertHandle = options.insertHandle,
            removeHandle = options.removeHandle,
            selectHandle = options.selectHandle,
            gHeight,


            split = function (val) {
                return val.split(/,\s*/);
            },

            extractLast = function (term) {
                return split(term).pop();
            };

        $zip
            // handle keydown and paste event
            .on("keydown", function (event) {
                if ($zip.parent('td')) {
                    $zip.parent('td').removeClass('DialogError');
                }
                if (event.keyCode === $.ui.keyCode.TAB && $(this).data('uiAutocomplete').menu.activeMenu.is(':visible')) {
                    event.preventDefault();
                }
                if ((event.keyCode >= 48 && event.keyCode <= 57) || (event.keyCode >= 96 && event.keyCode <= 105)) {
                    insertHandle();
                }
            })
            // handle keydown and paste event
            .on("paste", function (event) {
                insertHandle();
            })
            // handle change event of input to decide if zip was removed
            .on("change", function (event, remove) {
                // check if change was triggered by callback (onAfterReset) from inputaction.js
                if (remove) {
                    removeHandle();
                }
            })
            .autocomplete({
                delay: 300,
                source: function (request, response) {
                    $.ajax({
                        'dataType': 'json',
                        'type': 'POST',
                        'data': {
                            'ViewAction': 'JSONViewPresenceByZipCode',
                            'ObjectID': options.taxMatrixId,
                            'ZipCode': request.term
                        }
                    })
                        .done(function (data, status, jqXHR) {
                            response(data);
                        });
                },
                search: function () {
                    // custom minLength
                    var term = extractLast(this.value);
                    if (term.length < 3) {
                        // close menu (autocompletion list)
                        $(this).data('uiAutocomplete').menu.activeMenu.hide();
                        return false;
                    }
                },
                focus: function () {
                    // prevent value inserted on focus
                    return false;
                },
                select: function (event, ui) {
                    selectHandle(ui);

                    return false;
                }
            });

        if ($zip.data('uiAutocomplete')) {
            // format data for output in autocomplete list
            $zip.data('uiAutocomplete')._renderItem = function (ul, item) {
                return $("<li>")
                    .append("<a class='TruncateText'>" + item.ZipCode + ' - ' + item.City + ', ' + item.County + ', ' + item.State + "</a>")
                    .appendTo(ul);
            };

            // fit height of autocompletion list if content is lower as the default height
            $zip.data('uiAutocomplete')._resizeMenu = function () {
                var menu = this.menu.element,
                    list = menu.find('li');

                // get default height of autocompletion list when opening it the first time
                if (!gHeight) {
                    gHeight = menu.height();
                }

                // fit menu width to the width of the input field
                menu.css({
                    "min-width" : $zip.innerWidth()
                });
                // assign height of list items if their height is lower than default height
                menu.height((gHeight > list.eq(0).outerHeight(true) * list.length) ? list.eq(0).outerHeight(true) * list.length : gHeight);
            };
        }
    };

    return {
        // delete physical presence
        taxusDeletePresence: function (selector, taxMatrixId) {
            $(selector).on('click', function (evt) {
                var _this = $(this),
                    form = _this.closest('form');

                ep.ajax({
                    'dataType': 'json',
                    'type': 'POST',
                    'data': {
                        'ChangeAction': 'JSONDeleteTaxAreaUS',
                        'PresenceID': _this.data('objectid'),
                        'ObjectID': taxMatrixId
                    }
                })
                    // uncomment the fail method for debug use
                    // .fail(function (jqXHR, status, error) {
                    //     console.log("fail:", error);
                    // })
                    .done(function (data, status, jqXHR) {
                        // fade out parent table row
                        _this.closest('tr').fadeOut({
                            // complete: function () {
                            //     // trigger change event to activate save button
                            //     form.trigger('change');
                            // }
                        });
                    });

            });
        },

        taxusAddPresence: function (options) {
            var $zip = options.zip,
                $hidden = {
                    // Zip2TaxCity,
                    // Zip2TaxState,
                    // Zip2TaxZipCode
                },
                addHiddenInputs = function (address) {
                    var form = $zip.closest('form');

                    $hidden.Zip2TaxCity = $('<input type="hidden" name="Zip2TaxCity" />');
                    $hidden.Zip2TaxCity.val(address.City);
                    form.append($hidden.Zip2TaxCity);

                    $hidden.Zip2TaxState = $('<input type="hidden" name="Zip2TaxState" />');
                    $hidden.Zip2TaxState.val(address.State);
                    form.append($hidden.Zip2TaxState);

                    $hidden.Zip2TaxZipCode = $('<input type="hidden" name="Zip2TaxZipCode" />');
                    $hidden.Zip2TaxZipCode.val(address.ZipCode);
                    form.append($hidden.Zip2TaxZipCode);

                },

                removeHiddenInputs = function () {
                    // remove hidden inputs from DOM
                    if ($hidden.Zip2TaxCity || $hidden.Zip2TaxState || $hidden.Zip2TaxZipCode) {
                        $.each($hidden, function (index, elem) {
                            elem.remove();
                            $hidden[index] = undefined;
                            delete $hidden[index];
                        });
                    }
                },

                opt = $.extend({
                    insertHandle: function () {
                        removeHiddenInputs();
                    },
                    removeHandle: function () {
                        removeHiddenInputs();
                    },
                    selectHandle:  function (ui) {
                        // add hidden inputs for (ZipCode, State, City)
                        addHiddenInputs(ui.item);

                        $zip[0].value = ui.item.ZipCode + ' - ' + ui.item.City + ', ' + ui.item.County + ', ' + ui.item.State;
                    },
                }, options);


            getAddressFromZip(opt);
        },

        wizardAddAddress: function (options) {
            var $zip = $(options.zip.selector),
                $cityAndState = options.cityAndState,
                $city = options.spanCity,
                $state = options.spanState,
                $hidden = {
                    city: $(options.city.selector),
                    state: $(options.state.selector)
                },
                setHiddenInputs = function (address) {
                    if (address) {
                        $hidden.city.val(address.City);
                        $hidden.state.val(address.State);

                        // Set correct text
                        $city.text(address.City);
                        $state.text(address.State);
                        $cityAndState.removeClass('TextWeaker');
                    } else {
                        // Clear everything
                        $hidden.city.val('');
                        $hidden.state.val('');

                        $city.text('');
                        $state.text('');
                        $cityAndState.addClass('TextWeaker');
                    }
                },
                opt = $.extend(options, {
                    insertHandle: function () {
                        setHiddenInputs();
                    },
                    removeHandle: function () {
                        setHiddenInputs();
                    },
                    selectHandle:  function (ui) {
                        // set hidden inputs for State and City
                        setHiddenInputs(ui.item);

                        $zip[0].value = ui.item.ZipCode;
                    },
                    zip: $zip
                });


            getAddressFromZip(opt);
        }

    };

});

